@@ -40,6 +40,7 @@
 #include "asterisk/file.h"
 
 #include <assert.h>
+#include <sys/stat.h>
 #include <linux/limits.h>
 #include <openssl/evp.h>
 
@@ -105,6 +106,8 @@ AST_TEST_DEFINE(crypto_rsa_encrypt)
 
 	ast_test_status_update(test, "Executing RSA encryption test\n");
 
+	ast_test_capture_init(&cap);
+
 	if (!ast_check_command_in_path(command)) {
 		ast_test_status_update(test, "couldn't find %s\n", command);
 		return res;
@@ -115,6 +118,9 @@ AST_TEST_DEFINE(crypto_rsa_encrypt)
 	push_key_dir((const char *)key_dir);
 	snprintf(priv, sizeof(priv), "%s/%s.key", key_dir, keypair1);
 
+	/* because git doesn't preserve permissions */
+	(void)chmod(priv, 0400);
+
 	if (ast_crypto_reload() != 1) {
 		ast_test_status_update(test, "Couldn't force crypto reload\n");
 		goto cleanup;
@@ -190,6 +196,8 @@ AST_TEST_DEFINE(crypto_rsa_decrypt)
 
 	ast_test_status_update(test, "Executing RSA decryption test\n");
 
+	ast_test_capture_init(&cap);
+
 	if (!ast_check_command_in_path(command)) {
 		ast_test_status_update(test, "couldn't find %s\n", command);
 		return res;
@@ -285,6 +293,8 @@ AST_TEST_DEFINE(crypto_sign)
 
 	ast_test_status_update(test, "Executing RSA signing test\n");
 
+	ast_test_capture_init(&cap);
+
 	if (!ast_check_command_in_path(command)) {
 		ast_test_status_update(test, "couldn't find %s\n", command);
 		return res;
@@ -396,6 +406,8 @@ AST_TEST_DEFINE(crypto_verify)
 
 	ast_test_status_update(test, "Executing RSA signature verification test\n");
 
+	ast_test_capture_init(&cap);
+
 	if (!ast_check_command_in_path(command)) {
 		ast_test_status_update(test, "couldn't find %s\n", command);
 		return res;
@@ -406,6 +418,9 @@ AST_TEST_DEFINE(crypto_verify)
 	push_key_dir((const char *)key_dir);
 	snprintf(priv, sizeof(priv), "%s/%s.key", key_dir, keypair1);
 
+	/* because git doesn't preserve permissions */
+	(void)chmod(priv, 0400);
+
 	if (ast_crypto_reload() != 1) {
 		ast_test_status_update(test, "Couldn't force crypto reload\n");
 		goto cleanup;
@@ -491,6 +506,8 @@ AST_TEST_DEFINE(crypto_aes_encrypt)
 
 	ast_test_status_update(test, "Executing AES-ECB encryption test\n");
 
+	ast_test_capture_init(&cap);
+
 	if (!ast_check_command_in_path(command)) {
 		ast_test_status_update(test, "couldn't find %s\n", command);
 		return res;
@@ -565,6 +582,8 @@ AST_TEST_DEFINE(crypto_aes_decrypt)
 
 	ast_test_status_update(test, "Executing AES-ECB decryption test\n");
 
+	ast_test_capture_init(&cap);
+
 	if (!ast_check_command_in_path(command)) {
 		ast_test_status_update(test, "couldn't find %s\n", command);
 		return res;
